@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)db_load.c	4.38 (Berkeley) 3/2/91";
-static char rcsid[] = "$Id: db_load.c,v 8.30 1996/11/11 06:36:49 vixie Exp $";
+static char rcsid[] = "$Id: db_load.c,v 8.33 1998/05/11 04:19:45 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -323,6 +323,13 @@ db_load(filename, in_origin, zp, def_domain)
   					n = n * 10 + (*cp++ - '0');
 				}
 				while (isdigit(*cp));
+				if (*cp != '\0') {
+					errs++;
+					syslog(LOG_INFO,
+					       "%s: Line %d: bad TTL: %s.\n",
+					       filename, lineno, buf);
+					break;
+				}
 				if (zp->z_type == Z_CACHE) {
 				    /* this allows the cache entry to age */
 				    /* while sitting on disk (powered off) */
@@ -913,11 +920,12 @@ db_load(filename, in_origin, zp, def_domain)
 					nk = 1;		/* No-key */
 				else
 					nk = 0;		/* have a key */
-
-				if ((KEYFLAG_NO_AUTH | KEYFLAG_ZONEKEY) ==
-				    (keyflags &
-				     (KEYFLAG_NO_AUTH | KEYFLAG_ZONEKEY))) {
-					SETERR("Zone key with no auth.");
+                              if ((keyflags & KEYFLAG_ZONEKEY) && 
+                                  (KEYFLAG_TYPE_CONF_ONLY ==
+                                   (keyflags & KEYFLAG_TYPEMASK))) {
+                                      /* Zone key must have Authentication bit
+                                       set  ogud@tis.com */ 
+                                      SETERR("Zonekey needs authentication bit");
 					goto err;
 				}
 
@@ -944,10 +952,8 @@ db_load(filename, in_origin, zp, def_domain)
 				   the algorithm-ID's that we know about. */
 				switch (al) {
 				case ALGORITHM_MD5RSA:
-					if (klen == 0) {
-						SETERR("No key for RSA algid");
-						goto err;
-					}
+					if (klen == 0)
+						break;
 					expstart = cp;
 					expbytes = *expstart++;
 					if (expbytes == 0)
@@ -1364,6 +1370,7 @@ db_load(filename, in_origin, zp, def_domain)
 			continue;
 
 		case ERROR:
+			SETERR("parser error");
 			break;
 		}
  err:
@@ -1989,7 +1996,7 @@ get_netlist(fp, netlistp, allow, print_tag)
 	char *print_tag;
 {
 	struct netinfo *ntp, **end;
-	char buf[BUFSIZ], *maskp;
+	char buf[MAXDNAME], *maskp;
 	struct in_addr ina;
 
 	for (end = netlistp; *end; end = &(**end).next)
