@@ -1,6 +1,6 @@
 #if !defined(lint) && !defined(SABER)
 static char sccsid[] = "@(#)ns_main.c	4.55 (Berkeley) 7/1/91";
-static char rcsid[] = "$Id: ns_main.c,v 8.23 1996/11/11 06:36:49 vixie Exp $";
+static char rcsid[] = "$Id: ns_main.c,v 8.26 1998/05/11 04:19:45 vixie Exp $";
 #endif /* not lint */
 
 /*
@@ -172,7 +172,7 @@ main(argc, argv, envp)
 	const int on = 1;
 	int rfd, size, len;
 	time_t lasttime, maxctime;
-	u_char buf[BUFSIZ];
+	u_char buf[PACKETSZ];
 #ifdef NeXT
 	int old_sigmask;
 #endif
@@ -300,7 +300,10 @@ main(argc, argv, envp)
 	n = 0;
 #if defined(DEBUG) && defined(LOG_PERROR)
 	if (debug)
-		n = LOG_PERROR;
+		n |= LOG_PERROR;
+#endif
+#ifdef LOG_NOWAIT
+	n |= LOG_NOWAIT;
 #endif
 #ifdef LOG_DAEMON
 	openlog("named", LOG_PID|LOG_CONS|LOG_NDELAY|n, LOGFAC);
@@ -830,7 +833,7 @@ main(argc, argv, envp)
 						  malloc(rbufsize))
 						) {
 						    sp->s_buf = buf;
-						    sp->s_size  = sizeof(buf);
+						    sp->s_bufsize=sizeof(buf);
 					    } else {
 						    sp->s_bufsize = rbufsize;
 					    }
@@ -863,6 +866,12 @@ main(argc, argv, envp)
 				sp->s_bufp += n;
 				sp->s_size -= n;
 			}
+
+			if (sp->s_size > 0 &&
+			    (n == -1) &&
+			    (errno == PORT_WOULDBLK))
+				continue;
+
 			/*
 			 * we don't have enough memory for the query.
 			 * if we have a query id, then we will send an
@@ -883,8 +892,10 @@ main(argc, argv, envp)
 				(void) writemsg(sp->s_rfd, sp->s_buf,
 						HFIXEDSZ);
 			    }
+			    sqrm(sp);
 			    continue;
 			}
+
 			/*
 			 * If the message is too short to contain a valid
 			 * header, try to send back an error, and drop the
@@ -905,10 +916,9 @@ main(argc, argv, envp)
 				(void) writemsg(sp->s_rfd, sp->s_buf,
 						HFIXEDSZ);
 			    }
+			    sqrm(sp);
 			    continue;
 			}
-			if ((n == -1) && (errno == PORT_WOULDBLK))
-				continue;
 			if (n <= 0) {
 				sqrm(sp);
 				continue;
@@ -1453,7 +1463,6 @@ sigprof()
 	dprintf(1, (ddt, "sigprof()\n"));
 	if (fork() == 0)
 	{
-		(void) chdir(_PATH_TMPDIR);
 		exit(1);
 	}
 	errno = save_errno;
