@@ -7,6 +7,7 @@
  * response to a query.
  */
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
@@ -115,9 +116,7 @@ validate(dname, qdomain, server, type, class, data, dlen
 	int exactmatch = 0;
 	struct fwdinfo *fwd;
 
-#ifdef	DATUMREFCNT
 	nsp[0] = NULL;
-#endif
 	dprintf(3, (ddt,
 		    "validate(), d:%s, s:[%s], t:%d, c:%d\n",
 		    dname, inet_ntoa(server->sin_addr), type, class));
@@ -154,9 +153,7 @@ validate(dname, qdomain, server, type, class, data, dlen
 		 * in either case, do not cache
 		 **/
 		dprintf(5, (ddt, "validate: auth data found\n"));
-#ifdef	DATUMREFCNT
 		free_nsp(nsp);
-#endif
 		if (needs_prime_cache)
 			needs_prime_cache = 0;
 
@@ -210,9 +207,7 @@ validate(dname, qdomain, server, type, class, data, dlen
 		/* stick_in_queue(dname, type, class, data); */
 		if (needs_prime_cache)
 			needs_prime_cache = 0;
-#ifdef	DATUMREFCNT
 		free_nsp(nsp);
-#endif
 		return (INVALID);
     
 	case OK: /*proceed */
@@ -221,21 +216,15 @@ validate(dname, qdomain, server, type, class, data, dlen
 			needs_prime_cache = 0;
 		if (samedomain(dname, qdomain) ||
 		    check_addr_ns(nsp, server, dname)) {
-#ifdef	DATUMREFCNT
 			free_nsp(nsp);
-#endif
 			return (VALID_CACHE);
 		}
 		/* server is not one of those we know of */
 		/* stick_in_queue(dname, type, class, data); */
-#ifdef	DATUMREFCNT
 		free_nsp(nsp);
-#endif
 		return (INVALID);
 	default:
-#ifdef	DATUMREFCNT
 		free_nsp(nsp);
-#endif
 		return (INVALID);
 	} /*switch*/
 
@@ -620,7 +609,7 @@ dovalidate(msg, msglen, rrp, zone, flags, qdomain, server, VCode)
 	u_int32_t ttl;
 	char dname[MAXDNAME];
 	u_char *cp1;
-	u_char data[BUFSIZ];
+	u_char data[MAXDNAME*2 + INT32SZ*5];
 	register HEADER *hp = (HEADER *) msg;
 
 	dprintf(2, (ddt, "dovalidate(zone %d, flags %x)\n",
